/*
 *  Calculation of Pi using quadrature realized with a fork/join approach with threads.
 *
 *  Copyright © 2009 Russel Winder
 */

object Pi_Scala_Threads extends  Application {
  def execute ( numberOfTasks : Int ) {
    val n = 1000000000l
    val delta = 1.0 / n
    val startTimeNanos = System.nanoTime ( )
    val sliceSize = n / numberOfTasks
    val threads = new Array[Thread] ( numberOfTasks )
    var sum = 0.0
    for ( i <- 0 until numberOfTasks ) {
      threads ( i ) = new Thread ( new Runnable ( ) {
          def run ( ) {
            var j = 1 + i * sliceSize
            val end = ( i + 1 ) * sliceSize
            var localSum = 0.0
            while ( j <= end ) {
              val x = ( j - 0.5 ) * delta
              localSum += 1.0 / ( 1.0 + x * x )
              j += 1
            }
            threads.synchronized { sum += localSum }
          }
        } )
    }
    threads.foreach { t => t.start ( ) }
    threads.foreach { t => t.join ( ) }
    val pi = 4.0 * sum * delta
    val elapseTime = ( System.nanoTime ( ) - startTimeNanos ) / 1e9
    println ( "==== Scala Threads pi = " + pi )
    println ( "==== Scala Threads iteration count = " + n )
    println ( "==== Scala Threads elapse = " + elapseTime )
    println ( "==== Scala Threads processor count = " + Runtime.getRuntime ( ).availableProcessors ( ) )
    println ( "==== Scala Threads thread count = " + numberOfTasks )
  }
  execute ( 1 )
  println ( )
  execute ( 2 )
  println ( )
  execute ( 8 )
  println ( )
  execute ( 32 )
}
