;  Calculation of Pi using quadrature. Parallel algorithm.
;
;  Copyright © 2009 Russel Winder

( defn partialSum [ parameter ]
  ( let [
         start ( get parameter 0 )
         end ( get parameter 1 )
         delta ( get parameter 2 )
         ]
    ( loop [
            i start
            sum 0.0
            ]
      ( if ( = i end )
        sum
        ( let [ x ( * ( - i 0.5 ) delta ) ]
          ( recur ( inc i ) ( + sum ( / 1.0 ( + 1.0 ( * x x ) ) ) ) ) ) ) ) ) )

( defn execute [ numberOfThreads ]
  ( let [
         n 100000000 ; 10 times fewer due to speed issues.
         delta ( / 1.0 n )
         startTimeNanos ( System/nanoTime )
         sliceSize ( / n numberOfThreads )
         slices ( for [ i ( range 0 numberOfThreads ) ] [  ( + 1 ( * i sliceSize ) )  ( * ( + i 1 ) sliceSize ) delta ] ) 
                                        ;
                                        ;  Using map here works but totally sequentializes the computation.
                                        ;  Using pmap here works and employs multiple threads but the
                                        ;  program fails to terminate for about a minute once all the
                                        ;  work is complete.
                                        ;
         pi ( * 4.0 ( reduce + ( pmap partialSum slices ) ) delta )
         elapseTime ( / ( - ( System/nanoTime ) startTimeNanos ) 1e9 )
         ]
    ( println ( str "==== Clojure Parallel Map pi = " pi ) )
    ( println ( str "==== Clojure Parallel Map iteration count = " n ) )
    ( println ( str "==== Clojure Parallel Map elapse = " elapseTime ) )
    ( println ( str "==== Clojure Parallel Map number of threads = " numberOfThreads ) )
    ( println ( str "==== Clojure Parallel Map number of processors = " ( .. Runtime getRuntime availableProcessors ) ) ) ) )

( execute 1 )
( println )
( execute 2 )
( println )
( execute 8 )
( println )
( execute 32 )
