--  Haskell implementation of Pi by Quadrature.  Parallel version using parMap.
--
--  Copyright (c) 2009 Russel Winder

module Main where

import Data.Time.Clock ( getCurrentTime , diffUTCTime )
import GHC.Conc ( numCapabilities )
import  Control.Parallel.Strategies ( rwhnf , parMap )

piIter :: Integer -> Integer -> Integer -> Double -> Double -> Double
piIter  n from to delta accumulator
    | n > to = accumulator * delta
    | otherwise = 
        let
          nPlus1 = n + 1
          x = ( ( fromIntegral n ) - 0.5 ) * delta
          value = accumulator + 4.0 / ( 1.0 + x * x )        
        in
          nPlus1 `seq` x `seq` value `seq` piIter nPlus1 from to delta value

piQuadSlice :: Double -> ( Integer , Integer ) -> Double
piQuadSlice delta sliceSpecification = piIter start start end delta 0.0
    where
      start = fst sliceSpecification
      end = snd sliceSpecification

execute :: Integer -> IO ( )
execute numberOfSlices = do
  let n = 100000000 -- 10 times fewer for testing purposes.
  let delta = 1.0 / ( fromIntegral n )
  startTime <- getCurrentTime
  let sliceSize = n `div` numberOfSlices
  let pi = sum ( parMap rwhnf ( piQuadSlice delta ) [ ( ( 1 +  i * sliceSize ) , ( ( i + 1 ) * sliceSize ) ) | i <- [ 0 .. ( numberOfSlices - 1 ) ] ] )
  --let pi = sum ( map ( piQuadSlice delta ) [ ( ( 1 +  i * sliceSize ) , ( ( i + 1 ) * sliceSize ) ) | i <- [ 0 .. ( numberOfSlices - 1 ) ] ] )
  --  Don't get the time here since nothing has actually been computed as yet since pi has not been used.
  print ( "==== Haskell ParMap pi = " ++ show pi )
  print ( "==== Haskell ParMap iteration count = " ++ show n )
  endTime <- getCurrentTime
  print ( "==== Haskell ParMap elapse time = " ++ show ( diffUTCTime endTime startTime ) )
  print ( "==== Haskell ParMap thread count = " ++ show numberOfSlices )
  print ( "==== Haskell ParMap processor count = " ++ show numCapabilities )

main :: IO ( )
main = do
  execute 1
  --print ""
  --execute 2
  --print ""
  --execute 8
  --print ""
  --execute 32
