#! /usr/bin/env python
# -*- mode: python; coding: utf-8 -*-

#  Calculation of Pi using quadrature.
#
#  Copyright © 2008-9 Russel Winder

import time
import pp

def processSlice ( start , end , delta ) :
    sum = 0.0
    #  NB Using "for i in range ( start , end + 1 )" here would be a very bad move since range creates a
    #  list and with numbers this big you really don't want to do that.
    i = start
    while i < end + 1 :
        x = ( i - 0.5 ) * delta
        sum += 1.0 / ( 1.0 + x * x )
        i += 1
    return sum

server = pp.Server ( )

n = 10000000 # 100 times fewer due to speed issues.
delta = 1.0 / n
startTime = time.time ( ) # time.clock ( )
nProcessors = server.get_ncpus ( )
slice = n / nProcessors
jobs = [ server.submit ( processSlice , ( 1 + i * slice , ( i + 1 ) * slice , delta ) ) for i in range ( 0, nProcessors ) ]
results = [ job ( ) for job in jobs ]
pi = 4.0 * sum ( results ) * delta
elapseTime = time.time ( ) - startTime # time.clock ( ) - startTime # Using clock gives a value close to 0 always!
print "==== Python PP pi =" , pi
print "==== Python PP iteration count =", n
print "==== Python PP elapse =" , elapseTime
print "==== Python PP processor count =" , nProcessors
server.print_stats ( )
