#! /usr/bin/env python
# -*- mode: python; coding: utf-8 -*-

#  Calculation of Pi using quadrature.  Using the processing package with one process per processor.
#
#  Copyright © 2008-9 Russel Winder

import time
import processing

def processSlice ( start , end , delta , output ) :
    sum = 0.0
    #  NB Using "for i in range ( start , end + 1 )" here would be a very bad move since range creates a
    #  list and with numbers this big you really don't want to do that.
    i = start
    while i < end + 1 :
        x = ( i - 0.5 ) * delta
        sum += 1.0 / ( 1.0 + x * x )
        i += 1
    output.put ( sum )
    output.close ( )

n = 10000000 # 100 times fewer due to speed issues.
delta = 1.0 / n
startTime = time.time ( ) # time.clock ( )
nProcessors = processing.cpuCount ( )
slice = n / nProcessors
resultsQueue = processing.Queue ( )
processes = [ processing.Process ( target = processSlice , args = ( 1 + i * slice , ( i + 1 ) * slice , delta , resultsQueue ) ) for i in range ( 0 , nProcessors ) ]
for p in processes : p.start ( )
for p in processes : p.join ( )
results = [ ]
times = [ ]
while not resultsQueue.empty ( ) :
    s = resultsQueue.get ( )
    results.append ( s )
pi = 4.0 * sum ( results ) * delta
elapseTime = time.time ( ) - startTime # time.clock ( ) - startTime # Using clock gives a value close to 0 always!
print "==== Python Processing Process pi =" , pi
print "==== Python Processing Process iteration count =", n
print "==== Python Processing Process elapse =" , elapseTime
print "==== Python Processing Process processor count =" , nProcessors
